﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace RoboRace
{
    public partial class Interactive : Form
    {
        Connector connector;
        GameState currentState;
        Image wall, blank, treasure, hole;
        Image movingU, movingR, movingL, movingD;
        Image robotU, robotR, robotL, robotD;

        public Interactive(Connector c)
        {
            connector = c;
            currentState = connector.getCurrentState();
            InitializeComponent();
        }

        private void pbMap_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            for (int i = 0; i < currentState.map.Length; i++)
            {
                for (int j = 0; j < currentState.map[i].Length; j++)
                {
                    //if (!drawing) break;
                    switch (currentState.map[i][j])
                    {
                        case '.':
                            g.DrawImage(blank, j * 32, i * 32, 32, 32);
                            break;
                        case 'X':
                            g.DrawImage(wall, j * 32, i * 32, 32, 32);
                            break;
                        case '#':
                            g.DrawImage(hole, j * 32, i * 32, 32, 32);
                            break;
                        case '$':
                            g.DrawImage(treasure, j * 32, i * 32, 32, 32);
                            break;
                        case '^':
                            g.DrawImage(movingU, j * 32, i * 32, 32, 32);
                            break;
                        case 'v':
                            g.DrawImage(movingD, j * 32, i * 32, 32, 32);
                            break;
                        case '<':
                            g.DrawImage(movingL, j * 32, i * 32, 32, 32);
                            break;
                        case '>':
                            g.DrawImage(movingR, j * 32, i * 32, 32, 32);
                            break;
                    }
                }
            }
            switch(currentState.robotRotation)
            {
                case 0:
                    g.DrawImage(robotU, currentState.robotX * 32, currentState.robotY*32, 32, 32);
                    break;
                case 1:
                    g.DrawImage(robotR, currentState.robotX * 32, currentState.robotY * 32, 32, 32);
                    break;
                case 2:
                    g.DrawImage(robotD, currentState.robotX * 32, currentState.robotY * 32, 32, 32);
                    break;
                case 3:
                    g.DrawImage(robotL, currentState.robotX * 32, currentState.robotY * 32, 32, 32);
                    break;
            }

            lbMoves.Text = string.Join("", currentState.possibleMoves.ToArray());
        }

        private Image getRotatedImage(Image initial, int num)
        {
            Image newImage = (Image)initial.Clone();
            if (num == 1) newImage.RotateFlip(RotateFlipType.Rotate90FlipNone);
            if (num == 2) newImage.RotateFlip(RotateFlipType.Rotate180FlipNone);
            if (num == 3) newImage.RotateFlip(RotateFlipType.Rotate270FlipNone);
            return newImage;
        }

        private void Interactive_Load(object sender, EventArgs e)
        {
            wall = Image.FromFile("images/zed.png");
            hole = Image.FromFile("images/dira.png");
            treasure = Image.FromFile("images/poklad.png");
            blank = Image.FromFile("images/free.png");
            movingU = Image.FromFile("images/pas.png");
            movingR = getRotatedImage(movingU, 1);
            movingD = getRotatedImage(movingU, 2);
            movingL = getRotatedImage(movingU, 3);
            robotU = Image.FromFile("images/robot.png");
            robotR = getRotatedImage(robotU, 1);
            robotD = getRotatedImage(robotU, 2);
            robotL = getRotatedImage(robotU, 3);
        }

        private void btPreview_Click(object sender, EventArgs e)
        {
            currentState = connector.getCurrentState();
            if (GameLogic.checkValidity(currentState, tbMove.Text, true))
            {
                currentState = GameLogic.patchState(currentState, tbMove.Text, false);
            }
            else
            {
                MessageBox.Show("Tento tah není platný, rozbil bych se.");
            }
            pbMap.Refresh();
        }

        private void btSend_Click(object sender, EventArgs e)
        {
            currentState = connector.getCurrentState();
            if (GameLogic.checkValidity(currentState, tbMove.Text, false))
            {
                connector.sendMoves(tbMove.Text);
            }
            else
            {
                MessageBox.Show("Tento tah by vedl k předčasnému ukončení hry, proto nebude proveden. Pokud hru chcete ukončit, stačí zavřít příslušná okna.");
            }
            currentState = connector.getCurrentState();
            if (currentState.robotX == currentState.endX && currentState.robotY == currentState.endY)
            {
                MessageBox.Show("Gratuluji, vyhrál jste!");
                btPreview.Enabled = false;
                btSend.Enabled = false;
                btSolve.Enabled = false;
            }
            pbMap.Refresh();
        }

        private void btSolve_Click(object sender, EventArgs e)
        {
            string s = tbRequest.Text;
            string[] ss = s.Split(' ');
            if (ss.Length < 3) return;
            int targetX = int.Parse(ss[0]), targetY = int.Parse(ss[1]);
            int targetRotation = int.Parse(ss[2]);
            string res = GameLogic.findMoves(currentState, targetX-1, targetY-1, targetRotation);
            if (res == "")
            {
                MessageBox.Show("Sled kroků se nepodařilo nalézt.");
            }
            else
            {
                MessageBox.Show("Sled kroků k požadované pozici bude předvyplněn do políčka k odeslání.");
                tbMove.Text = res;
            }
        }
    }
}
